#ifndef PARTITIONIMAGE_H
#define PARTITIONIMAGE_H

#include <QObject>
#include <QDateTime>
#include <QFile>
#include <QTextStream>

class PartitionImage : public QObject
{
    Q_OBJECT
private:
    QString id;
    QString parentId;
    QString label;
    QString path;
    QString fileExtension;
    int size;
    bool isEncrypted;
    QString encryptedAesKey;
    QString aesKeyEncryptionIv;
    QString encryptedInitVector;
    QString initVectorEncryptionIv;
    QDateTime createdAt;
    QFile imageFile;
public:
    explicit PartitionImage(const QString &uniqueId, const QString &inputPath, const QString &inputFileExtension, QObject *parent = nullptr);

    QString getId() const;
    void setId(const QString &value);
    QString getParentId() const;
    void setParentId(const QString &value);
    QString getLabel() const;
    void setLabel(const QString &value);
    QString getPath() const;
    void setPath(const QString &value);
    int getSize() const;
    void setSize(int value);
    QDateTime getCreatedAt() const;
    void setCreatedAt(const QDateTime &value);
    QString getEncryptedAesKey() const;
    void setEncryptedAesKey(const QString &value, const bool &isEncrypted, const QString &masterAesKey = "");
    QString getEncryptedInitVector() const;
    void setEncryptedInitVector(const QString &value, const bool &isEncrypted, const QString &masterAesKey = "");

    virtual qint64 encrypt(const QString &masterAesKey);
    virtual qint64 decrypt(const QString &masterAesKey, const QString &initVector);
    virtual int setPrivileges(const QString &suPassword, int octalPermissions, QTextStream *outputStream = nullptr);
    virtual int uploadToServer(const QString &sshPrivKeyPath, const QString &userId, const QString &apiKey, QTextStream *outputStream = nullptr);
    QString getFileExtension() const;
    void setFileExtension(const QString &value);
};

#endif // PARTITIONIMAGE_H
